@@ -1,9 +1,5 @@
 <?php
 /**
- *
- *
- * Created on December 12, 2007
- *
  * Copyright © 2007 Roan Kattouw "<Firstname>.<Lastname>@gmail.com"
  *
  * This program is free software; you can redistribute it and/or modify
@@ -200,6 +196,6 @@ class ApiQueryAllCategories extends ApiQueryGeneratorBase {
 	}
 
 	public function getHelpUrls() {
-		return 'https://www.mediawiki.org/wiki/API:Allcategories';
+		return 'https://www.mediawiki.org/wiki/Special:MyLanguage/API:Allcategories';
 	}
 }
