@@ -1,9 +1,5 @@
 <?php
 /**
- *
- *
- * Created on Jun 20, 2007
- *
  * Copyright © 2007 Roan Kattouw "<Firstname>.<Lastname>@gmail.com"
  *
  * This program is free software; you can redistribute it and/or modify
@@ -52,7 +48,7 @@ class ApiRollback extends ApiBase {
 
 		// If change tagging was requested, check that the user is allowed to tag,
 		// and the tags are valid
-		if ( count( $params['tags'] ) ) {
+		if ( $params['tags'] ) {
 			$tagStatus = ChangeTags::canAddTagsAccompanyingChange( $params['tags'], $user );
 			if ( !$tagStatus->isOK() ) {
 				$this->dieStatus( $tagStatus );
@@ -69,24 +65,8 @@ class ApiRollback extends ApiBase {
 			$params['tags']
 		);
 
-		// We don't care about multiple errors, just report one of them
 		if ( $retval ) {
-			if ( isset( $retval[0][0] ) &&
-				( $retval[0][0] == 'alreadyrolled' || $retval[0][0] == 'cantrollback' )
-			) {
-				$error = $retval[0];
-				$userMessage = $this->msg( $error[0], array_slice( $error, 1 ) );
-				// dieUsageMsg() doesn't support $extraData
-				$errorCode = $error[0];
-				$errorInfo = isset( ApiBase::$messageMap[$errorCode] ) ?
-					ApiBase::$messageMap[$errorCode]['info'] :
-					$errorCode;
-				$this->dieUsage( $errorInfo, $errorCode, 0, [
-					'messageHtml' => $userMessage->parseAsBlock()
-				] );
-			}
-
-			$this->dieUsageMsg( reset( $retval ) );
+			$this->dieStatus( $this->errorArrayToStatus( $retval, $user ) );
 		}
 
 		$watch = 'preferences';
@@ -108,17 +88,6 @@ class ApiRollback extends ApiBase {
 			'last_revid' => intval( $details['target']->getID() )
 		];
 
-		$oldUser = $details['current']->getUserText( Revision::FOR_THIS_USER );
-		$lastUser = $details['target']->getUserText( Revision::FOR_THIS_USER );
-		$diffUrl = $titleObj->getFullURL( [
-			'diff' => $info['revid'],
-			'oldid' => $info['old_revid'],
-			'diffonly' => '1'
-		] );
-		$info['messageHtml'] = $this->msg( 'rollback-success-notify' )
-			->params( $oldUser, $lastUser, $diffUrl )
-			->parseAsBlock();
-
 		$this->getResult()->addValue( null, $this->getModuleName(), $info );
 	}
 
@@ -181,7 +150,7 @@ class ApiRollback extends ApiBase {
 			? $params['user']
 			: User::getCanonicalName( $params['user'] );
 		if ( !$this->mUser ) {
-			$this->dieUsageMsg( [ 'invaliduser', $params['user'] ] );
+			$this->dieWithError( [ 'apierror-invaliduser', wfEscapeWikiText( $params['user'] ) ] );
 		}
 
 		return $this->mUser;
@@ -202,17 +171,17 @@ class ApiRollback extends ApiBase {
 		if ( isset( $params['title'] ) ) {
 			$this->mTitleObj = Title::newFromText( $params['title'] );
 			if ( !$this->mTitleObj || $this->mTitleObj->isExternal() ) {
-				$this->dieUsageMsg( [ 'invalidtitle', $params['title'] ] );
+				$this->dieWithError( [ 'apierror-invalidtitle', wfEscapeWikiText( $params['title'] ) ] );
 			}
 		} elseif ( isset( $params['pageid'] ) ) {
 			$this->mTitleObj = Title::newFromID( $params['pageid'] );
 			if ( !$this->mTitleObj ) {
-				$this->dieUsageMsg( [ 'nosuchpageid', $params['pageid'] ] );
+				$this->dieWithError( [ 'apierror-nosuchpageid', $params['pageid'] ] );
 			}
 		}
 
 		if ( !$this->mTitleObj->exists() ) {
-			$this->dieUsageMsg( 'notanarticle' );
+			$this->dieWithError( 'apierror-missingtitle' );
 		}
 
 		return $this->mTitleObj;
@@ -229,6 +198,6 @@ class ApiRollback extends ApiBase {
 	}
 
 	public function getHelpUrls() {
-		return 'https://www.mediawiki.org/wiki/API:Rollback';
+		return 'https://www.mediawiki.org/wiki/Special:MyLanguage/API:Rollback';
 	}
 }
