@@ -34,6 +34,11 @@
  */
 class TextContent extends AbstractContent {
 
+	/**
+	 * @var string
+	 */
+	protected $mText;
+
 	/**
 	 * @param string $text
 	 * @param string $model_id
@@ -231,7 +236,7 @@ class TextContent extends AbstractContent {
 	 *
 	 * @param Title $title Context title for parsing
 	 * @param int $revId Revision ID (for {{REVISIONID}})
-	 * @param ParserOptions $options Parser options
+	 * @param ParserOptions $options
 	 * @param bool $generateHtml Whether or not to generate HTML
 	 * @param ParserOutput &$output The output object to fill (reference).
 	 */
