@@ -148,7 +148,7 @@ abstract class File implements IDBAccessObject {
 	protected $isSafeFile;
 
 	/** @var string Required Repository class type */
-	protected $repoClass = 'FileRepo';
+	protected $repoClass = FileRepo::class;
 
 	/** @var array Cache of tmp filepaths pointing to generated bucket thumbnails, keyed by width */
 	protected $tmpBucketedThumbCache = [];
@@ -250,7 +250,7 @@ abstract class File implements IDBAccessObject {
 		$oldMime = $old->getMimeType();
 		$n = strrpos( $new, '.' );
 		$newExt = self::normalizeExtension( $n ? substr( $new, $n + 1 ) : '' );
-		$mimeMagic = MimeMagic::singleton();
+		$mimeMagic = MediaWiki\MediaWikiServices::getInstance()->getMimeAnalyzer();
 
 		return $mimeMagic->isMatchingExtension( $newExt, $oldMime );
 	}
@@ -268,7 +268,7 @@ abstract class File implements IDBAccessObject {
 	 * a two-part name, set the minor type to 'unknown'.
 	 *
 	 * @param string $mime "text/html" etc
-	 * @return array ("text", "html") etc
+	 * @return string[] ("text", "html") etc
 	 */
 	public static function splitMime( $mime ) {
 		if ( strpos( $mime, '/' ) !== false ) {
@@ -569,7 +569,7 @@ abstract class File implements IDBAccessObject {
 	 * format does not support that sort of thing, returns
 	 * an empty array.
 	 *
-	 * @return array
+	 * @return string[]
 	 * @since 1.23
 	 */
 	public function getAvailableLanguages() {
@@ -581,6 +581,25 @@ abstract class File implements IDBAccessObject {
 		}
 	}
 
+	/**
+	 * Get the language code from the available languages for this file that matches the language
+	 * requested by the user
+	 *
+	 * @param string $userPreferredLanguage
+	 * @return string|null
+	 */
+	public function getMatchedLanguage( $userPreferredLanguage ) {
+		$handler = $this->getHandler();
+		if ( $handler && method_exists( $handler, 'getMatchedLanguage' ) ) {
+			return $handler->getMatchedLanguage(
+				$userPreferredLanguage,
+				$handler->getAvailableLanguages( $this )
+			);
+		} else {
+			return null;
+		}
+	}
+
 	/**
 	 * In files that support multiple language, what is the default language
 	 * to use if none specified.
@@ -1404,7 +1423,7 @@ abstract class File implements IDBAccessObject {
 	 * Get all thumbnail names previously generated for this file
 	 * STUB
 	 * Overridden by LocalFile
-	 * @return array
+	 * @return string[]
 	 */
 	function getThumbnails() {
 		return [];
@@ -1445,7 +1464,9 @@ abstract class File implements IDBAccessObject {
 		// Purge cache of all pages using this file
 		$title = $this->getTitle();
 		if ( $title ) {
-			DeferredUpdates::addUpdate( new HTMLCacheUpdate( $title, 'imagelinks' ) );
+			DeferredUpdates::addUpdate(
+				new HTMLCacheUpdate( $title, 'imagelinks', 'file-purge' )
+			);
 		}
 	}
 
@@ -1453,9 +1474,9 @@ abstract class File implements IDBAccessObject {
 	 * Return a fragment of the history of file.
 	 *
 	 * STUB
-	 * @param int $limit Limit of rows to return
-	 * @param string $start Only revisions older than $start will be returned
-	 * @param string $end Only revisions newer than $end will be returned
+	 * @param int|null $limit Limit of rows to return
+	 * @param string|int|null $start Only revisions older than $start will be returned
+	 * @param string|int|null $end Only revisions newer than $end will be returned
 	 * @param bool $inc Include the endpoints of the time range
 	 *
 	 * @return File[]
@@ -2078,9 +2099,9 @@ abstract class File implements IDBAccessObject {
 	 *   File::FOR_PUBLIC       to be displayed to all users
 	 *   File::FOR_THIS_USER    to be displayed to the given user
 	 *   File::RAW              get the description regardless of permissions
-	 * @param User $user User object to check for, only if FOR_THIS_USER is
+	 * @param User|null $user User object to check for, only if FOR_THIS_USER is
 	 *   passed to the $audience parameter
-	 * @return string
+	 * @return null|string
 	 */
 	function getDescription( $audience = self::FOR_PUBLIC, User $user = null ) {
 		return null;
@@ -2140,7 +2161,7 @@ abstract class File implements IDBAccessObject {
 	 * field of this file, if it's marked as deleted.
 	 * STUB
 	 * @param int $field
-	 * @param User $user User object to check, or null to use $wgUser
+	 * @param User|null $user User object to check, or null to use $wgUser
 	 * @return bool
 	 */
 	function userCan( $field, User $user = null ) {
@@ -2156,7 +2177,7 @@ abstract class File implements IDBAccessObject {
 	}
 
 	/**
-	 * @return array HTTP header name/value map to use for HEAD/GET request responses
+	 * @return string[] HTTP header name/value map to use for HEAD/GET request responses
 	 * @since 1.30
 	 */
 	function getContentHeaders() {
@@ -2165,7 +2186,7 @@ abstract class File implements IDBAccessObject {
 			$metadata = $this->getMetadata();
 
 			if ( is_string( $metadata ) ) {
-				$metadata = MediaWiki\quietCall( 'unserialize', $metadata );
+				$metadata = Wikimedia\quietCall( 'unserialize', $metadata );
 			}
 
 			if ( !is_array( $metadata ) ) {
