@@ -86,11 +86,11 @@ class CachedBagOStuff extends HashBagOStuff {
 		return $this->backend->deleteObjectsExpiringBefore( $date, $progressCallback );
 	}
 
-	public function makeKey() {
+	public function makeKey( $class, $component = null ) {
 		return call_user_func_array( [ $this->backend, __FUNCTION__ ], func_get_args() );
 	}
 
-	public function makeGlobalKey() {
+	public function makeGlobalKey( $class, $component = null ) {
 		return call_user_func_array( [ $this->backend, __FUNCTION__ ], func_get_args() );
 	}
 
