@@ -40,10 +40,6 @@ class ProfilerStub extends Profiler {
 	public function close() {
 	}
 
-	public function getCurrentSection() {
-		return '';
-	}
-
 	public function logData() {
 	}
 
