@@ -222,9 +222,8 @@ class SpecialEditTags extends UnlistedSpecialPage {
 		$numRevisions = 0;
 		// Live revisions...
 		$list = $this->getList();
-		// @codingStandardsIgnoreStart Generic.CodeAnalysis.ForLoopWithTestFunctionCall.NotAllowed
+		// phpcs:ignore Generic.CodeAnalysis.ForLoopWithTestFunctionCall
 		for ( $list->reset(); $list->current(); $list->next() ) {
-			// @codingStandardsIgnoreEnd
 			$item = $list->current();
 			$numRevisions++;
 			$out->addHTML( $item->getHTML() );
@@ -240,6 +239,9 @@ class SpecialEditTags extends UnlistedSpecialPage {
 
 		// Show form if the user can submit
 		if ( $this->isAllowed ) {
+			$conf = $this->getConfig();
+			$oldCommentSchema = $conf->get( 'CommentTableSchemaMigrationStage' ) === MIGRATION_OLD;
+
 			$form = Xml::openElement( 'form', [ 'method' => 'post',
 					'action' => $this->getPageTitle()->getLocalURL( [ 'action' => 'submit' ] ),
 					'id' => 'mw-revdel-form-revisions' ] ) .
@@ -252,12 +254,14 @@ class SpecialEditTags extends UnlistedSpecialPage {
 						Xml::label( $this->msg( 'tags-edit-reason' )->text(), 'wpReason' ) .
 					'</td>' .
 					'<td class="mw-input">' .
-						Xml::input(
-							'wpReason',
-							60,
-							$this->reason,
-							[ 'id' => 'wpReason', 'maxlength' => 100 ]
-						) .
+						Xml::input( 'wpReason', 60, $this->reason, [
+							'id' => 'wpReason',
+							// HTML maxlength uses "UTF-16 code units", which means that characters outside BMP
+							// (e.g. emojis) count for two each. This limit is overridden in JS to instead count
+							// Unicode codepoints (or 255 UTF-8 bytes for old schema).
+							// "- 155" is to leave room for the auto-generated part of the log entry.
+							'maxlength' => $oldCommentSchema ? 100 : CommentStore::COMMENT_CHARACTER_LIMIT - 155,
+						] ) .
 					'</td>' .
 				"</tr><tr>\n" .
 					'<td></td>' .
@@ -310,9 +314,8 @@ class SpecialEditTags extends UnlistedSpecialPage {
 			// Otherwise, use a multi-select field for adding tags, and a list of
 			// checkboxes for removing them
 
-			// @codingStandardsIgnoreStart Generic.CodeAnalysis.ForLoopWithTestFunctionCall.NotAllowed
+			// phpcs:ignore Generic.CodeAnalysis.ForLoopWithTestFunctionCall
 			for ( $list->reset(); $list->current(); $list->next() ) {
-				// @codingStandardsIgnoreEnd
 				$currentTags = $list->current()->getTags();
 				if ( $currentTags ) {
 					$tags = array_merge( $tags, explode( ',', $currentTags ) );
@@ -451,9 +454,8 @@ class SpecialEditTags extends UnlistedSpecialPage {
 	 */
 	protected function failure( $status ) {
 		$this->getOutput()->setPageTitle( $this->msg( 'actionfailed' ) );
-		$this->getOutput()->addWikiText( '<div class="errorbox">' .
-			$status->getWikiText( 'tags-edit-failure' ) .
-			'</div>'
+		$this->getOutput()->addWikiText(
+			Html::errorBox( $status->getWikiText( 'tags-edit-failure' ) )
 		);
 		$this->showForm();
 	}
