@@ -36,11 +36,6 @@ class SpecialPasswordReset extends FormSpecialPage {
 	/** @var PasswordReset */
 	private $passwordReset = null;
 
-	/**
-	 * @var string[] Temporary storage for the passwords which have been sent out, keyed by username.
-	 */
-	private $passwords = [];
-
 	/**
 	 * @var Status
 	 */
