@@ -8,7 +8,7 @@
  *
  */
 
-$digitTransformTable = array(
+$digitTransformTable = [
 	'0' => '༠', # &#x0f20;
 	'1' => '༡', # &#x0f21;
 	'2' => '༢', # &#x0f22;
@@ -19,5 +19,4 @@ $digitTransformTable = array(
 	'7' => '༧', # &#x0f27;
 	'8' => '༨', # &#x0f28;
 	'9' => '༩', # &#x0f29;
-);
-
+];
