@@ -11,7 +11,7 @@
 $fallback = 'fr';
 
 $bookstoreList = [
-	'Amazon.fr'    => 'http://www.amazon.fr/exec/obidos/ISBN=$1',
+	'Amazon.fr'    => 'https://www.amazon.fr/exec/obidos/ISBN=$1',
 	'alapage.fr'   => 'http://www.alapage.com/mx/?tp=F&type=101&l_isbn=$1&donnee_appel=ALASQ&devise=&',
 	'fnac.com'     => 'http://www3.fnac.com/advanced/book.do?isbn=$1',
 	'chapitre.com' => 'http://www.chapitre.com/frame_rec.asp?isbn=$1',
@@ -302,4 +302,3 @@ $dateFormats = [
 ];
 
 $separatorTransformTable = [ ',' => "\xc2\xa0", '.' => ',' ];
-
