@@ -18,20 +18,11 @@
  * @author Urhixidur
  */
 
-/**
- * Бұл қазақша тілдесуінің жерсіндіру файлы
- *
- * ШЕТКІ ПАЙДАЛАНУШЫЛАР: БҰЛ ФАЙЛДЫ ТІКЕЛЕЙ ӨҢДЕМЕҢІЗ!
- *
- * Бұл файлдағы өзгерістер бағдарламалық жасақтама кезекті жаңартылғанда жоғалтылады.
- * Уикиде өз бапталымдарыңызды істей аласыз.
- * Әкімші боп кіргеніңізде, [[Арнайы:Барлық хабарлар]] деген бетке өтіңіз де
- * мында тізімделінген МедиаУики:* сипаты бар беттерді өңдеңіз.
- */
 $separatorTransformTable = [
 	',' => "\xc2\xa0",
 	'.' => ',',
 ];
+$minimumGroupingDigits = 2;
 
 $fallback8bitEncoding = 'windows-1251';
 
