@@ -29,6 +29,9 @@ $digitTransformTable = [
 	'7' => '٧', # &#x0667;
 	'8' => '٨', # &#x0668;
 	'9' => '٩', # &#x0669;
-	'.' => '٫', # &#x066b; wrong table ?
+];
+
+$separatorTransformTable = [
+	'.' => '٫', # &#x066b;
 	',' => '٬', # &#x066c;
 ];
