@@ -12,6 +12,24 @@
 
 $rtl = true;
 
+$digitTransformTable = [
+	'0' => '۰', # &#x1776;
+	'1' => '۱', # &#x1777;
+	'2' => '۲', # &#x1778;
+	'3' => '۳', # &#x1779;
+	'4' => '۴', # &#x1780;
+	'5' => '۵', # &#x1781;
+	'6' => '۶', # &#x1782;
+	'7' => '۷', # &#x1783;
+	'8' => '۸', # &#x1784;
+	'9' => '۹', # &#x1785;
+];
+
+$separatorTransformTable = [
+	'.' => '٫', # &#x066b;
+	',' => '٬', # &#x066c;
+];
+
 $namespaceNames = [
 	NS_MEDIA            => 'رسنۍ',
 	NS_SPECIAL          => 'ځانگړی',
