@@ -1,9 +1,9 @@
 /*!
  * JavaScript for Special:Preferences: Check for successbox to replace with notifications.
  */
-( function ( mw, $ ) {
+( function ( $ ) {
 	$( function () {
 		var convertmessagebox = require( 'mediawiki.notification.convertmessagebox' );
 		convertmessagebox();
 	} );
-}( mediaWiki, jQuery ) );
+}( jQuery ) );
