@@ -54,6 +54,8 @@ class HooksTest extends MediaWikiTestCase {
 	 */
 	public function testOldStyleHooks( $msg, array $hook, $expectedFoo, $expectedBar ) {
 		global $wgHooks;
+
+		$this->hideDeprecated( 'wfRunHooks' );
 		$foo = $bar = 'original';
 
 		$wgHooks['MediaWikiHooksTest001'][] = $hook;
@@ -67,6 +69,7 @@ class HooksTest extends MediaWikiTestCase {
 	 * @dataProvider provideHooks
 	 * @covers Hooks::register
 	 * @covers Hooks::run
+	 * @covers Hooks::callHook
 	 */
 	public function testNewStyleHooks( $msg, $hook, $expectedFoo, $expectedBar ) {
 		$foo = $bar = 'original';
@@ -83,6 +86,7 @@ class HooksTest extends MediaWikiTestCase {
 	 * @covers Hooks::register
 	 * @covers Hooks::getHandlers
 	 * @covers Hooks::run
+	 * @covers Hooks::callHook
 	 */
 	public function testNewStyleHookInteraction() {
 		global $wgHooks;
@@ -122,6 +126,7 @@ class HooksTest extends MediaWikiTestCase {
 	/**
 	 * @expectedException MWException
 	 * @covers Hooks::run
+	 * @covers Hooks::callHook
 	 */
 	public function testUncallableFunction() {
 		Hooks::register( 'MediaWikiHooksTest001', 'ThisFunctionDoesntExist' );
@@ -130,6 +135,7 @@ class HooksTest extends MediaWikiTestCase {
 
 	/**
 	 * @covers Hooks::run
+	 * @covers Hooks::callHook
 	 */
 	public function testFalseReturn() {
 		Hooks::register( 'MediaWikiHooksTest001', function ( &$foo ) {
@@ -147,6 +153,7 @@ class HooksTest extends MediaWikiTestCase {
 
 	/**
 	 * @covers Hooks::runWithoutAbort
+	 * @covers Hooks::callHook
 	 */
 	public function testRunWithoutAbort() {
 		$list = [];
@@ -169,6 +176,7 @@ class HooksTest extends MediaWikiTestCase {
 
 	/**
 	 * @covers Hooks::runWithoutAbort
+	 * @covers Hooks::callHook
 	 */
 	public function testRunWithoutAbortWarning() {
 		Hooks::register( 'MediaWikiHooksTest001', function ( &$foo ) {
