@@ -163,7 +163,7 @@ class TitleTest extends MediaWikiTestCase {
 	 */
 	public function testSecureAndSplitValid( $text ) {
 		$this->secureAndSplitGlobals();
-		$this->assertInstanceOf( 'Title', Title::newFromText( $text ), "Valid: $text" );
+		$this->assertInstanceOf( Title::class, Title::newFromText( $text ), "Valid: $text" );
 	}
 
 	/**
@@ -282,7 +282,6 @@ class TitleTest extends MediaWikiTestCase {
 	/**
 	 * Auth-less test of Title::isValidMoveOperation
 	 *
-	 * @group Database
 	 * @param string $source
 	 * @param string $target
 	 * @param array|string|bool $expected Required error
@@ -435,7 +434,7 @@ class TitleTest extends MediaWikiTestCase {
 		$this->setContentLang( $contLang );
 
 		$title = Title::newFromText( $titleText );
-		$this->assertInstanceOf( 'Title', $title,
+		$this->assertInstanceOf( Title::class, $title,
 			"Test must be passed a valid title text, you gave '$titleText'"
 		);
 		$this->assertEquals( $expected,
@@ -553,6 +552,7 @@ class TitleTest extends MediaWikiTestCase {
 	}
 
 	/**
+	 * @covers Title::newFromTitleValue
 	 * @dataProvider provideNewFromTitleValue
 	 */
 	public function testNewFromTitleValue( TitleValue $value ) {
@@ -573,6 +573,7 @@ class TitleTest extends MediaWikiTestCase {
 	}
 
 	/**
+	 * @covers Title::getTitleValue
 	 * @dataProvider provideGetTitleValue
 	 */
 	public function testGetTitleValue( $text ) {
@@ -604,6 +605,7 @@ class TitleTest extends MediaWikiTestCase {
 	}
 
 	/**
+	 * @covers Title::getFragment
 	 * @dataProvider provideGetFragment
 	 *
 	 * @param string $full
@@ -912,4 +914,55 @@ class TitleTest extends MediaWikiTestCase {
 	public function testGetPrefixedDBKey( Title $title, $expected ) {
 		$this->assertEquals( $expected, $title->getPrefixedDBkey() );
 	}
+
+	/**
+	 * @covers Title::getFragmentForURL
+	 * @dataProvider provideGetFragmentForURL
+	 *
+	 * @param string $titleStr
+	 * @param string $expected
+	 */
+	public function testGetFragmentForURL( $titleStr, $expected ) {
+		$this->setMwGlobals( [
+			'wgFragmentMode' => [ 'html5' ],
+			'wgExternalInterwikiFragmentMode' => 'legacy',
+		] );
+		$dbw = wfGetDB( DB_MASTER );
+		$dbw->insert( 'interwiki',
+			[
+				[
+					'iw_prefix' => 'de',
+					'iw_url' => 'http://de.wikipedia.org/wiki/',
+					'iw_api' => 'http://de.wikipedia.org/w/api.php',
+					'iw_wikiid' => 'dewiki',
+					'iw_local' => 1,
+					'iw_trans' => 0,
+				],
+				[
+					'iw_prefix' => 'zz',
+					'iw_url' => 'http://zzwiki.org/wiki/',
+					'iw_api' => 'http://zzwiki.org/w/api.php',
+					'iw_wikiid' => 'zzwiki',
+					'iw_local' => 0,
+					'iw_trans' => 0,
+				],
+			],
+			__METHOD__,
+			[ 'IGNORE' ]
+		);
+
+		$title = Title::newFromText( $titleStr );
+		self::assertEquals( $expected, $title->getFragmentForURL() );
+
+		$dbw->delete( 'interwiki', '*', __METHOD__ );
+	}
+
+	public function provideGetFragmentForURL() {
+		return [
+			[ 'Foo', '' ],
+			[ 'Foo#ümlåût', '#ümlåût' ],
+			[ 'de:Foo#Bå®', '#Bå®' ],
+			[ 'zz:Foo#тест', '#.D1.82.D0.B5.D1.81.D1.82' ],
+		];
+	}
 }
