@@ -8,11 +8,6 @@
  * @covers ApiUnblock
  */
 class ApiUnblockTest extends ApiTestCase {
-	protected function setUp() {
-		parent::setUp();
-		$this->doLogin();
-	}
-
 	/**
 	 * @expectedException ApiUsageException
 	 */
@@ -22,10 +17,7 @@ class ApiUnblockTest extends ApiTestCase {
 				'action' => 'unblock',
 				'user' => 'UTApiBlockee',
 				'reason' => 'Some reason',
-			],
-			null,
-			false,
-			self::$users['sysop']->getUser()
+			]
 		);
 	}
 }
