@@ -5,13 +5,10 @@
  * @group Database
  * @group medium
  * @todo This test suite is severly broken and need a full review
+ *
+ * @covers ApiWatch
  */
 class ApiWatchTest extends ApiTestCase {
-	protected function setUp() {
-		parent::setUp();
-		$this->doLogin();
-	}
-
 	function getTokens() {
 		return $this->getTokenList( self::$users['sysop'] );
 	}
