@@ -1,7 +1,7 @@
 <?php
 
 /**
- * @author Adam Shorland
+ * @author Addshore
  *
  * @group Diff
  */
@@ -11,7 +11,7 @@ class DiffTest extends MediaWikiTestCase {
 	 * @covers Diff::getEdits
 	 */
 	public function testGetEdits() {
-		$obj = new Diff( array(), array() );
+		$obj = new Diff( [], [] );
 		$obj->edits = 'FooBarBaz';
 		$this->assertEquals( 'FooBarBaz', $obj->getEdits() );
 	}
