@@ -1,5 +1,7 @@
 <?php
 
+use MediaWiki\MediaWikiServices;
+
 /**
  * @group JobQueue
  * @group medium
@@ -26,7 +28,7 @@ class JobQueueTest extends MediaWikiTestCase {
 			}
 			$baseConfig = $wgJobTypeConf[$name];
 		} else {
-			$baseConfig = [ 'class' => 'JobQueueDB' ];
+			$baseConfig = [ 'class' => JobQueueDBSingle::class ];
 		}
 		$baseConfig['type'] = 'null';
 		$baseConfig['wiki'] = wfWikiID();
@@ -232,7 +234,7 @@ class JobQueueTest extends MediaWikiTestCase {
 
 		$j = $queue->pop();
 		// Make sure ack() of the twin did not delete the sibling data
-		$this->assertType( 'NullJob', $j );
+		$this->assertType( NullJob::class, $j );
 	}
 
 	/**
@@ -381,3 +383,11 @@ class JobQueueTest extends MediaWikiTestCase {
 			[ 'lives' => 0, 'usleep' => 0, 'removeDuplicates' => 1, 'i' => $i ] + $rootJob );
 	}
 }
+
+class JobQueueDBSingle extends JobQueueDB {
+	protected function getDB( $index ) {
+		$lb = MediaWikiServices::getInstance()->getDBLoadBalancer();
+		// Override to not use CONN_TRX_AUTOCOMMIT so that we see the same temporary `job` table
+		return $lb->getConnection( $index, [], $this->wiki );
+	}
+}
