@@ -5,10 +5,13 @@
  * @group Media
  * @group medium
  *
- * @todo covers tags
+ * @covers BitmapHandler
  */
 class ExifRotationTest extends MediaWikiMediaTestCase {
 
+	/** @var BitmapHandler */
+	private $handler;
+
 	protected function setUp() {
 		parent::setUp();
 		$this->checkPHPExtension( 'exif' );
