@@ -184,7 +184,7 @@ class BotPasswordSessionProviderTest extends MediaWikiTestCase {
 	public function testNewSessionInfoForRequest() {
 		$provider = $this->getProvider();
 		$user = static::getTestSysop()->getUser();
-		$request = $this->getMockBuilder( 'FauxRequest' )
+		$request = $this->getMockBuilder( \FauxRequest::class )
 			->setMethods( [ 'getIP' ] )->getMock();
 		$request->expects( $this->any() )->method( 'getIP' )
 			->will( $this->returnValue( '127.0.0.1' ) );
@@ -212,7 +212,7 @@ class BotPasswordSessionProviderTest extends MediaWikiTestCase {
 		$provider->setLogger( $logger );
 
 		$user = static::getTestSysop()->getUser();
-		$request = $this->getMockBuilder( 'FauxRequest' )
+		$request = $this->getMockBuilder( \FauxRequest::class )
 			->setMethods( [ 'getIP' ] )->getMock();
 		$request->expects( $this->any() )->method( 'getIP' )
 			->will( $this->returnValue( '127.0.0.1' ) );
@@ -264,7 +264,7 @@ class BotPasswordSessionProviderTest extends MediaWikiTestCase {
 		], $logger->getBuffer() );
 		$logger->clearBuffer();
 
-		$request2 = $this->getMockBuilder( 'FauxRequest' )
+		$request2 = $this->getMockBuilder( \FauxRequest::class )
 			->setMethods( [ 'getIP' ] )->getMock();
 		$request2->expects( $this->any() )->method( 'getIP' )
 			->will( $this->returnValue( '10.0.0.1' ) );
