@@ -2,6 +2,7 @@
 
 namespace MediaWiki\Session;
 
+use Config;
 use MediaWikiTestCase;
 use User;
 use Wikimedia\TestingAccessWrapper;
@@ -14,9 +15,16 @@ use Wikimedia\TestingAccessWrapper;
 class SessionBackendTest extends MediaWikiTestCase {
 	const SESSIONID = 'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa';
 
+	/** @var SessionManager */
 	protected $manager;
+
+	/** @var Config */
 	protected $config;
+
+	/** @var SessionProvider */
 	protected $provider;
+
+	/** @var TestBagOStuff */
 	protected $store;
 
 	protected $onSessionMetadataCalled = false;
@@ -25,6 +33,7 @@ class SessionBackendTest extends MediaWikiTestCase {
 	 * Returns a non-persistent backend that thinks it has at least one session active
 	 * @param User|null $user
 	 * @param string $id
+	 * @return SessionBackend
 	 */
 	protected function getBackend( User $user = null, $id = null ) {
 		if ( !$this->config ) {
@@ -142,14 +151,14 @@ class SessionBackendTest extends MediaWikiTestCase {
 		$this->assertSame( self::SESSIONID, $backend->getId() );
 		$this->assertSame( $id, $backend->getSessionId() );
 		$this->assertSame( $this->provider, $backend->getProvider() );
-		$this->assertInstanceOf( 'User', $backend->getUser() );
+		$this->assertInstanceOf( User::class, $backend->getUser() );
 		$this->assertSame( 'UTSysop', $backend->getUser()->getName() );
 		$this->assertSame( $info->wasPersisted(), $backend->isPersistent() );
 		$this->assertSame( $info->wasRemembered(), $backend->shouldRememberUser() );
 		$this->assertSame( $info->forceHTTPS(), $backend->shouldForceHTTPS() );
 
 		$expire = time() + 100;
-		$this->store->setSessionMeta( self::SESSIONID, [ 'expires' => $expire ], 2 );
+		$this->store->setSessionMeta( self::SESSIONID, [ 'expires' => $expire ] );
 
 		$info = new SessionInfo( SessionInfo::MIN_PRIORITY, [
 			'provider' => $this->provider,
@@ -164,7 +173,7 @@ class SessionBackendTest extends MediaWikiTestCase {
 		$this->assertSame( self::SESSIONID, $backend->getId() );
 		$this->assertSame( $id, $backend->getSessionId() );
 		$this->assertSame( $this->provider, $backend->getProvider() );
-		$this->assertInstanceOf( 'User', $backend->getUser() );
+		$this->assertInstanceOf( User::class, $backend->getUser() );
 		$this->assertTrue( $backend->getUser()->isAnon() );
 		$this->assertSame( $info->wasPersisted(), $backend->isPersistent() );
 		$this->assertSame( $info->wasRemembered(), $backend->shouldRememberUser() );
@@ -258,7 +267,7 @@ class SessionBackendTest extends MediaWikiTestCase {
 	public function testResetId() {
 		$id = session_id();
 
-		$builder = $this->getMockBuilder( 'DummySessionProvider' )
+		$builder = $this->getMockBuilder( \DummySessionProvider::class )
 			->setMethods( [ 'persistsSessionId', 'sessionIdWasReset' ] );
 
 		$this->provider = $builder->getMock();
@@ -294,7 +303,7 @@ class SessionBackendTest extends MediaWikiTestCase {
 	}
 
 	public function testPersist() {
-		$this->provider = $this->getMockBuilder( 'DummySessionProvider' )
+		$this->provider = $this->getMockBuilder( \DummySessionProvider::class )
 			->setMethods( [ 'persistSession' ] )->getMock();
 		$this->provider->expects( $this->once() )->method( 'persistSession' );
 		$backend = $this->getBackend();
@@ -314,7 +323,7 @@ class SessionBackendTest extends MediaWikiTestCase {
 	}
 
 	public function testUnpersist() {
-		$this->provider = $this->getMockBuilder( 'DummySessionProvider' )
+		$this->provider = $this->getMockBuilder( \DummySessionProvider::class )
 			->setMethods( [ 'unpersistSession' ] )->getMock();
 		$this->provider->expects( $this->once() )->method( 'unpersistSession' );
 		$backend = $this->getBackend();
@@ -367,7 +376,7 @@ class SessionBackendTest extends MediaWikiTestCase {
 	public function testSetUser() {
 		$user = static::getTestSysop()->getUser();
 
-		$this->provider = $this->getMockBuilder( 'DummySessionProvider' )
+		$this->provider = $this->getMockBuilder( \DummySessionProvider::class )
 			->setMethods( [ 'canChangeUser' ] )->getMock();
 		$this->provider->expects( $this->any() )->method( 'canChangeUser' )
 			->will( $this->returnValue( false ) );
@@ -498,7 +507,7 @@ class SessionBackendTest extends MediaWikiTestCase {
 			->setMethods( [ 'onSessionMetadata' ] )->getMock();
 		$neverHook->expects( $this->never() )->method( 'onSessionMetadata' );
 
-		$builder = $this->getMockBuilder( 'DummySessionProvider' )
+		$builder = $this->getMockBuilder( \DummySessionProvider::class )
 			->setMethods( [ 'persistSession', 'unpersistSession' ] );
 
 		$neverProvider = $builder->getMock();
@@ -746,7 +755,7 @@ class SessionBackendTest extends MediaWikiTestCase {
 		$testData = [ 'foo' => 'foo!', 'bar', [ 'baz', null ] ];
 
 		// Not persistent
-		$this->provider = $this->getMockBuilder( 'DummySessionProvider' )
+		$this->provider = $this->getMockBuilder( \DummySessionProvider::class )
 			->setMethods( [ 'persistSession' ] )->getMock();
 		$this->provider->expects( $this->never() )->method( 'persistSession' );
 		$this->onSessionMetadataCalled = false;
@@ -772,7 +781,7 @@ class SessionBackendTest extends MediaWikiTestCase {
 		$this->assertNotEquals( 0, $wrap->expires );
 
 		// Persistent
-		$this->provider = $this->getMockBuilder( 'DummySessionProvider' )
+		$this->provider = $this->getMockBuilder( \DummySessionProvider::class )
 			->setMethods( [ 'persistSession' ] )->getMock();
 		$this->provider->expects( $this->atLeastOnce() )->method( 'persistSession' );
 		$this->onSessionMetadataCalled = false;
@@ -799,7 +808,7 @@ class SessionBackendTest extends MediaWikiTestCase {
 		$this->assertNotEquals( 0, $wrap->expires );
 
 		// Not persistent, not expiring
-		$this->provider = $this->getMockBuilder( 'DummySessionProvider' )
+		$this->provider = $this->getMockBuilder( \DummySessionProvider::class )
 			->setMethods( [ 'persistSession' ] )->getMock();
 		$this->provider->expects( $this->never() )->method( 'persistSession' );
 		$this->onSessionMetadataCalled = false;
@@ -891,7 +900,7 @@ class SessionBackendTest extends MediaWikiTestCase {
 		$manager->globalSessionRequest = $request;
 
 		session_id( self::SESSIONID );
-		\MediaWiki\quietCall( 'session_start' );
+		\Wikimedia\quietCall( 'session_start' );
 		$_SESSION['foo'] = __METHOD__;
 		$backend->resetId();
 		$this->assertNotEquals( self::SESSIONID, $backend->getId() );
@@ -929,7 +938,7 @@ class SessionBackendTest extends MediaWikiTestCase {
 		$manager->globalSessionRequest = $request;
 
 		session_id( self::SESSIONID . 'x' );
-		\MediaWiki\quietCall( 'session_start' );
+		\Wikimedia\quietCall( 'session_start' );
 		$backend->unpersist();
 		$this->assertSame( self::SESSIONID . 'x', session_id() );
 
@@ -940,7 +949,7 @@ class SessionBackendTest extends MediaWikiTestCase {
 	}
 
 	public function testGetAllowedUserRights() {
-		$this->provider = $this->getMockBuilder( 'DummySessionProvider' )
+		$this->provider = $this->getMockBuilder( \DummySessionProvider::class )
 			->setMethods( [ 'getAllowedUserRights' ] )
 			->getMock();
 		$this->provider->expects( $this->any() )->method( 'getAllowedUserRights' )
