@@ -1,11 +1,8 @@
 // From http://webdriver.io/guide/testrunner/pageobjects.html
 'use strict';
 class Page {
-	constructor() {
-		this.title = 'My Page';
-	}
 	open( path ) {
-		browser.url( '/index.php?title=' + path );
+		browser.url( browser.options.baseUrl + '/index.php?title=' + path );
 	}
 }
 module.exports = Page;
